@@ -1,23 +1,19 @@
 from telegrambotclient.api import (DEFAULT_API_HOST, TelegramBotAPI,
                                    TelegramBotAPIException)
-from telegrambotclient.base import TelegramBotException, Update
+from telegrambotclient.base import TelegramBotException
 from telegrambotclient.bot import TelegramBot
 from telegrambotclient.router import TelegramRouter
 from telegrambotclient.storage import TelegramStorage
 
 
 class TelegramBotClient:
-    __slots__ = ("_bot_data", "_router_data", "_name", "_bot_api_data")
+    __slots__ = ("_bot_data", "_router_data", "name", "_bot_api_data")
 
     def __init__(self, name: str = "default-client") -> None:
         self._bot_data = {}
         self._router_data = {}
         self._bot_api_data = {}
-        self._name = name
-
-    @property
-    def name(self):
-        return self._name
+        self.name = name
 
     def router(self, name: str = "default-router") -> TelegramRouter:
         router = self._router_data.get(name, None)
@@ -51,12 +47,12 @@ class TelegramBotClient:
     def bot(self, token: str) -> TelegramBot:
         return self._bot_data.get(token, None)
 
-    async def dispatch(self, token: str, raw_update):
-        simple_bot = self._bot_data.get(token, None)
-        if simple_bot is None:
+    async def dispatch(self, token: str, update):
+        bot = self.bot(token)
+        if bot is None:
             raise TelegramBotException(
-                "No bot found with token: '{0}'".format(token))
-        await simple_bot.dispatch(Update(**raw_update))
+                "There is not a bot with token: '{0}'".format(token))
+        await bot.dispatch(update)
 
 
 # default bot proxy
